﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FetchAIP
{
    class Crawl
    {
        public Queue<pdfLinks> CrawlAIP(BackgroundWorker bwCrawler, string baseURL, Queue<subPages> subpages)
        {
            List<string> visitedPages = new List<string>();
            Queue<subPages> subs = new Queue<subPages>(subpages);
            Queue<pdfLinks> pdfs = new Queue<pdfLinks>();

            int pagesCrawled = 0;

            while (!bwCrawler.CancellationPending && subs.Count > 0)
            {
                // Remove page URL being crawled from queue
                var page = subs.Dequeue();

                // Check if page URL being crawled has been visited
                if (visitedPages.Contains(page.PageURL))
                    continue;

                // Add page URL to visited list
                visitedPages.Add(page.PageURL);

                // Download the data to a buffer.
                Byte[] pageData;
                using (var client = new WebClient())
                {
                    pageData = client.DownloadData(baseURL + page.PageURL);
                }

                string pageHTML = Encoding.ASCII.GetString(pageData);

                // Load page in to html document
                var doc = new HtmlAgilityPack.HtmlDocument();
                doc.LoadHtml(pageHTML);

                // Crawl page for all links
                var links = doc.DocumentNode.SelectNodes("//a[@href]");

                foreach (var link in links)
                {
                    // If link contains ".pdf", add to pdfLinks list
                    if (link.Attributes["href"].Value.Contains(".pdf"))
                    {
                        pdfs.Enqueue(new pdfLinks
                        {
                            FileName = link.InnerText,
                            FileURL = link.Attributes["href"].Value.Trim().Replace("&#xD;&#xA;", string.Empty) // Cleanse pdf link
                        });
                    }

                    // If crawling the Home page, don't look for subpages
                    if (page.PageURL != "")
                    {
                        // If link contains "NavWalk.aspx", add to subPages queue
                        if (link.Attributes["href"].Value.Contains("NavWalk.aspx"))
                        {
                            subs.Enqueue(new subPages
                            {
                                PageName = link.InnerText,
                                PageURL = link.Attributes["href"].Value.Replace("&amp;", "&") // Cleanse subpage link
                            }); 
                        }
                    }
                }
                pagesCrawled++;
                
                // Update status with page being crawled
                bwCrawler.ReportProgress(pagesCrawled);
            }
            return pdfs;
        }
    }
}
